%% DtN for an annular cavity with a collocation point on each step
clear

% 0 < R1 < R2 < Rinf < Inf
R1   = 1;               % inner wall
R2   = 2;               % outer wall
Rinf = 4;               % far-field boundary of shallow region

% 0 < Hs < Hd <= Inf
Hs   = 0.05;            % depth of shallow region
Hd   = 1.3;             % depth of deep region  

m  = 0;     % angular number (>= 0)
Ns = 512;   % number of points for r in (0, Rinf)

% wavenumbers on the surface
ks = zerobess('J', m, Ns)/Rinf;

% give f in Hankel space - test function
kF = 2*pi; % desired wavenumber
[~, kF_ind] = min(abs(ks - kF));
fm = zeros(Ns, 1); fm(kF_ind) = 1;

%% Determine collocation points (do not modify)
Hdiff = Hd - Hs;        % difference in depths
DR    = R2 - R1;        % channel width
Rmid  = (R1 + R2)/2;    % middle of channel
eps_an = (R2-Rmid)/Rmid;% measure of channel width to channel radius

% empirical for determining innermost collocation point (depending on m)
if      m <= 15; jj = 1;
elseif  m <= 31; jj = 2;
elseif  m <= 40; jj = 3;
elseif  m <= 52; jj = 4;
else             jj = 5; %#okay
end
z = zerobess('DJ', m, jj); z = z(end)/ks(end);
if m == 0; minr = 0;
else minr = z; %#okay
end

% outermost collocation point
maxr = Rinf * (1 - 1/(2 * Ns));

% Enforce a collocation point on each step
% make sure that length of each interval is properly calculated
% determine the number of collocation points in each region
DRL = R1 - minr; DRR = maxr - R2;
Nd  = round(Ns * DR/(DRL + DR + DRR));
NsL = round((Ns - Nd) * DRL/(DRL + DRR));
NsR = Ns - (Nd + NsL);

% determine the collocation points
rd  = linspace(R1, R2, Nd+2); rd = rd(2:end-1);     % deep region
rsL = linspace(minr, R1, NsL);                      % shallow, inner
rsR = linspace(R2, maxr, NsR);                      % shallow, outer
rc  = [rsL, rd, rsR];                               % all shallow

disp(['R1 = ' num2str(R1) ', R2 = ' num2str(R2) ', Rinf = ' num2str(Rinf)])

%% make wavenumbers and eigenfunctions in deep geometry (do not modify)
get_wavenumbers % generates xi_{mp} and vartheta_{mp}

kd = xi_cell{m+1}(1:Nd)/Rmid;
td = t_cell{m+1}(1:Nd);

%% make meshes
% for matching condition
[kdm, rdm] = meshgrid(kd,rd);
[ksm, rsm] = meshgrid(ks,rc);
tdm = repmat(td, Nd, 1);

% cylindrical functions
Cyl = besselj(m, kdm .* rdm) .* cos(tdm * pi) + ...
    bessely(m, kdm .* rdm) .* sin(tdm * pi);

% if m = 0 then the first cylindrical function is the unit function
if m == 0
    Cyl(:,1) = ones(Nd, 1);
end

% for consistency condition at deep mesh points
[ksm_c, rsm_c] = meshgrid(ks, rd);

%% make matrices
% for matching condition
Asm = ksm .* coth(ksm * Hs) .* besselj(m, ksm .* rsm);

Adm = zeros(Ns, Nd);
Adm(NsL + (1:Nd), :) = kdm .* tanh(kdm * Hdiff) .* Cyl;

% consistency condition
Bsm = besselj(m, ksm_c .* rsm_c);
Bdm = Cyl;

% coefficient of f
Csm = ksm .* csch(ksm * Hs) .* besselj(m, ksm .* rsm);

A = [Asm, Adm; Bsm, -Bdm];
b = [Csm * fm; zeros(Nd,1)];
disp(['Condition number = ' num2str(cond(A), '%10.5e')])

%% LU decomposition with pivoting - precompute matrices to save time
[L,U,p] = lu(A,'vector');
gm_all = U\(L\b(p));
gms = gm_all(1:Ns);
gmd = gm_all((Ns+1):end);

%% Compute DtN for annulus, and two flat cylinders (for comparison)
DTNm  = ks .* (fm .* coth(ks * Hs) - gms .* csch(ks * Hs)); % annulus
DTNms = ks .* tanh(ks * Hs) .* fm;                          % depth Hs
DTNmd = ks .* tanh(ks * Hd) .* fm;                          % depth Hd

%% Plot results
bes_mats = besselj(m, ksm .* rsm);

DTN  = bes_mats * DTNm;     % annulus
DTNs = bes_mats * DTNms;    % shallow depth, Hs, everywhere
DTNd = bes_mats * DTNmd;    % deep depth, Hd, everywhere

figure
hold on
plot(rc, DTNs, 'b', 'LineWidth', 2)
plot(rc, DTNd, 'r', 'LineWidth', 2)
plot(rc, DTN,  'k', 'LineWidth', 2)
grid on; box on
plot(R1*[1 1], [-4 8], 'k--', 'LineWidth', 1) % inner wall
plot(R2*[1 1], [-4 8], 'k--', 'LineWidth', 1) % outer wall

legend('DtN shallow', 'DtN deep', 'DtN annulus', 'Location', 'NorthEast')
set(gca, 'FontSize', 18, 'FontName', 'Times New Roman')
xlabel('$$r$$', 'Interpreter', 'Latex', 'FontSize', 22)
