%% Compute library of cylinder functions with Neumann BCs
% note: might have convergence issues for large m as the function is very
% large near to its zeros (get 1e-6 error for m = 256)

% *** do not modify this code ***

% dimensionless width parameter in (0,1)
m_max = m;
num_s = Nd;            % number of number of roots for m = m_max 
s_max = m_max + num_s; % need 0 < m_max < s_max

L = (1 + eps_an)/(1 - eps_an); % L = lambda > 1 for eps_an > 0

% Bessel functions
Jm = @(m,x) besselj(m,x);
Ym = @(m,x) bessely(m,x);

% Bessel function derivative wrt x
dJm = @(m,x) 1/2 * (Jm(m-1, x) - Jm(m+1, x));
dYm = @(m,x) 1/2 * (Ym(m-1, x) - Ym(m+1, x));

d2Jm = @(m,x) 1/4 * (Jm(m-2, x) - 2*Jm(m, x) + Jm(m+2, x));
d2Ym = @(m,x) 1/4 * (Ym(m-2, x) - 2*Ym(m, x) + Ym(m+2, x));

%% find roots of f0(x) = J0(L * x) * Y0(x) - J0(x) * Y0(L * x)
% --- use auxiliary function given by Sorolla et al. (2013)

% define f0(x) and its derivatives with respect to x: df0(x) and d2f0(x)
f0   = @(x) Jm(0,L*x) .* Ym(0,x) - Jm(0,x) .* Ym(0,L*x);
df0  = @(x) L * (dJm(0,L*x) .* Ym(0,x) - Jm(0,x) .* dYm(0,L*x)) + ...
    (Jm(0,L*x) .* dYm(0,x) - dJm(0,x) .* Ym(0,L*x));
d2f0 = @(x) L^2 * (d2Jm(0,L*x) .* Ym(0,x) - Jm(0,x) .* d2Ym(0,L*x)) + ...
    2*L*(dJm(0,L*x) .* dYm(0,x) - dJm(0,x) .* dYm(0,L*x)) + ...
    (Jm(0,L*x) .* d2Ym(0,x) - d2Jm(0,x) .* Ym(0,L*x));

% define the auxiliary function F0(x) = f0(x)/df0(x)
F0  = @(x) f0(x)./df0(x);
dF0 = @(x) 1 - f0(x) .* d2f0(x)./(df0(x).^2);

% find roots using Newton's method
x0 = pi * (1:s_max)./(L-1); % initial guess using McMahon's approximation

ITER_max = 10;      % maximum number of iterations
TOL = 1e-14;        % function tolerance
cont = true; ii = 0;
while cont
    x0 = x0 - F0(x0)./dF0(x0);
    cont = logical(  (ii < ITER_max) .* any(abs(F0(x0) > TOL))  );
    ii = ii + 1;
end

%% find roots of f1(x) = J1(L * x) * Y1(x) - J1(x) * Y1(L * x)
% --- use interweaving method ---
options = optimset('TolFun', TOL);
x1 = zeros(1, s_max - 1);
for ss = 1:(s_max-1)
    x1(ss) = fzero(@(x)Jm(1,L*x).*Ym(1,x) - Jm(1,x).*Ym(1,L*x), ...
        [x0(ss) x0(ss+1)], options);   
end

%% determine roots of df0(x) = dJ0(L * x) * dY0(x) - dJ0(x) * dY0(L * x)
dx0 = [0 x1]; % root at 0 corresponds to unit eigenfunction

%% determine roots of dfm(x) = dJm(L * x) * dYm(x) - dJm(x) * dYm(L * x) for m >= 1
x_cell = cell(1, m_max+1);
x_cell{1} = dx0; % m = 0 case with s_max number of roots
for mm = 1:m_max
    disp(['m = ' num2str(mm) '/' num2str(m_max)])
    cc = mm + 1;
    dxm = zeros(1, s_max - mm);
    for ss = 1:(s_max - mm)
        left = max(x_cell{cc-1}(ss), 1e-4); % function blows-up at x = 0
        right = x_cell{cc-1}(ss+1);
        dxm(ss) = fzero(@(x)dJm(mm,L*x).*dYm(mm,x) - dJm(mm,x).*dYm(mm,L*x), ...
                [left right], options);
    end
    x_cell{cc} = dxm;
end

%% now define xi_cell and t_cell
% note: xi = x/(1 - eps_an), i.e. rescale back to variable so that R = 1 is
% center of channel
% note: we restrict t to the interval (0,1) as otherwise just multiply
% eigenfunctions by +/- 1.
xi_cell = cell(1, m_max+1); t_cell = cell(1, m_max+1);
for mm = 0:m_max
    cc = mm + 1;
    dxm = x_cell{cc}(1:num_s);
    xi_cell{cc} = dxm/(1 - eps_an);
    t_cell{cc} = mod(-1/pi * atan(dJm(mm,dxm)./dYm(mm,dxm)), 1);
end

%% check errors
max_error = zeros(1, m_max+1);
for mm = 0:m_max
    dxm = x_cell{mm+1}; 
    error = dJm(mm,L*dxm).*dYm(mm,dxm) - dJm(mm,dxm).*dYm(mm,L*dxm);
    max_error(mm+1) = max(abs(error));
end

