%% Produces corral DtN and example in Figure 3 of the paper
clear

m  = 1;       % angular number (>= 0)
Ns = 512;     % number of points for r in (0, Rinf)

% note: r mesh can be on r = Rinf as we have Neumann BCs
Rinf = 5; % outer radius
Rc   = 1; % inner radius

Hs    = 0.1;    % shallow depth (Hs < Hd)
Hd    = 1;      % deep depth
Hdiff = Hd - Hs;% difference in depths

% wavenumbers in shallow region
ks = zerobess('J', m, Ns)/Rinf;

% definition of first collocation point (minr)
if m == 0; minr = 0; else; minr = Rinf*ks(1)/ks(end); end
maxr = Rinf * (1 - 1/(2 * Ns)); % last collocation point

% define collocation points, rc
DR = maxr - minr;
Nd = round(Ns * (Rc - minr)/DR);
rd = linspace(minr, Rc, Nd + 1); rd = rd(1:end-1);
rs = linspace(Rc, maxr, Ns-Nd);
rc = [rd, rs];

% wavenumbers in deep region (includes zero for m == 0)
kd = zerobess('DJ', m, Nd)/Rc;

% give f in Hankel space - test function
fm = zeros(Ns, 1); fm(8) = 1; % *** CHANGE INPUT DATA ***

% make meshes -----------------------------------------------------------
% for matching condition
[kdm, rdm] = meshgrid(kd,rd);
[ksm, rsm] = meshgrid(ks,rc);

% for consistency condition
[ksm_c, rsm_c] = meshgrid(ks, rd);

% make matrices --------------------------------------------------------
% for matching condition
Asm = ksm .* coth(ksm * Hs) .* besselj(m, ksm .* rsm);

Adm = zeros(Ns, Nd);
Adm(1:Nd, :) = kdm .* tanh(kdm * Hdiff) .* besselj(m, kdm .* rdm);

% consistency condition
Bsm = besselj(m, ksm_c .* rsm_c);
Bdm = besselj(m, kdm .* rdm);

% coefficient of f
Csm = ksm .* csch(ksm * Hs) .* besselj(m, ksm .* rsm);

A = [Asm, Adm; Bsm, -Bdm];
b = [Csm * fm; zeros(Nd,1)];

%% LU decomposition with pivoting - precompute matrices to save time
[L,U,p] = lu(A,'vector');
gm_all = U\(L\b(p));

%% compute DtN maps
gm = gm_all(1:Ns);
DTNm  = ks .* (fm .* coth(ks * Hs) - gm .* csch(ks * Hs)); % DTN corral
DTNms = ks .* tanh(ks * Hs) .* fm;                  % DTN shallow cylinder
DTNmd = ks .* tanh(ks * Hd) .* fm;                  % DTN deep cylinder

%% plot results
bes_mat = besselj(m, ksm .* rsm);
f   = bes_mat * fm;     % input (Dirichlet) data
DTN = bes_mat * DTNm;   % DtN map for corral
DTNs= bes_mat * DTNms;  % DtN map for shallow cylinder
DTNd= bes_mat * DTNmd;  % DtN map for deep cylinder

maxv = 3.1;
minv = -2;

grey1 = 0.7 * [1 1 1];
grey2 = 0.4 * [1 1 1];

figure
set(gcf, 'Position', [400 400 800 400]) % left, bottom, width, height
lwd = 2.5;
% --- test function ---
subplot('Position', [0.07 0.68 0.53 0.28])
hold on
plot(Rc * [1 1], [-0.4,0.8], 'k--', 'LineWidth', 1)
plot([0 rc], [0; f], 'k', 'LineWidth', lwd + 1)
hold off
grid on
box on
axis tight
set(gca, 'XTick', 0:7, 'YTick', -0.4:0.2:0.8, 'FontName', ...
    'Times New Roman', 'FontSize', 16)
text(-0.6, 0.85, '$$(a)$$','Interpreter', 'Latex', 'FontSize', 20)
xticklabels({})
yticklabels({-0.4, [], 0, [], 0.4, [], 0.8})
text(3.4, 0.67, '$$\mbox{Dirichlet data}$$', 'Interpreter', 'Latex', ...
    'FontSize', 20)
text(0.25, 0.67, '$$\mbox{Deep}$$', 'Interpreter', 'Latex', ...
    'FontSize', 17)
text(1.18, 0.67, '$$\mbox{Shallow}$$', 'Interpreter', 'Latex', ...
    'FontSize', 17)

% --- DtN plots ---
subplot('Position', [0.07 0.1 0.53 0.53])
hold on
plot(Rc * [1 1], [minv,maxv], 'k--', 'LineWidth', 1)
plot([0 rc], [0; DTNs], '-', 'Color', grey1, 'LineWidth', lwd)
plot([0 rc], [0; DTNd], '-.', 'Color', grey2, 'LineWidth', lwd)
plot([0 rc], [0; DTN], 'k', 'LineWidth', lwd + 1)

x1 = 0.6; x2 = 1.4; y1 = -1.9; y2 = 0.8;
plot([x1 x2], [y1 y1], 'k')
plot([x1 x2], [y2 y2], 'k')
plot([x1 x1], [y1 y2], 'k')
plot([x2 x2], [y1 y2], 'k')

hold off
grid on
axis([0 5 minv maxv])
box on
set(gca, 'XTick', 0:7, 'YTick', -2:3, 'FontName', 'Times New Roman', 'FontSize', 16)
xlabel('$$r$$', 'Interpreter', 'Latex', 'FontSize', 20)
yticklabels({-2, -1, 0, 1, 2, 3})
text(-0.6, 3.1, '$$(b)$$','Interpreter', 'Latex', 'FontSize', 20)

text(3.4, 2.75, '$$\mbox{Neumann data}$$', 'Interpreter', 'Latex', ...
    'FontSize', 20)

% --- zoom-in ---
subplot('Position', [0.65 0.1 0.3 0.86])

% axes('Position', [0.4 0.45 0.4 0.2]) % left, bottom, width, height
hold on
plot(Rc * [1 1], [minv,maxv], 'k--', 'LineWidth', 1)
plot(rc, DTNs, '-', 'Color', grey1, 'LineWidth', lwd)
plot(rc, DTNd, '-.', 'Color', grey2, 'LineWidth', lwd)
plot(rc, DTN, 'k', 'LineWidth', lwd + 1)
axis([x1 x2 y1 y2])
box on
set(gca, 'XTick', x1:0.2:x2, 'YTick', -1.5:0.5:1, 'FontName', 'Times New Roman', ...
    'FontSize', 16)
grid on
text(0.5, 0.8, '$$(c)$$','Interpreter', 'Latex', 'FontSize', 20)
xlabel('$$r$$', 'Interpreter', 'Latex', 'FontSize', 20)