%% Compute the DtN of a symmetric channel for each wavenumber in y direction
% each step is at a grid point, but in the shallow region!
% domain size is x in [-Lx, Lx), y in [-Ly, Ly)
% channel is x in [-Lc, Lc), where Lc must be a grid point

clear

Hs = 0.2;               % depth of shallow region
Hd = 1;                 % depth of deep region
HD = Hd - Hs;           % difference in heights

Lx = 6;                 % half-width of shallow domain in x-direction
Ly = 6;                 % half-width of shallow domain in y-direction

N0 = 2^7;               % minimum number of Fourier modes
dx0 = Lx/N0;            % maximum dx, ensuring that step is always a grid point
Lc = 40 * dx0;          % ensure that step is at a collocation point

ky_ind = 1;             % y-frequency index             <--- change this!
fm_ind = 5;             % x-frequency index of input    <--- change this!

Nx = 2^8;               % number of Fourier modes in x-direction (Nx >= N0)
Ny = 2^8;               % number of Fourier modes in y-direction

disp(['Lx = ' num2str(Lx), ', Ly = ' num2str(Ly) , ', Lc = ' num2str(Lc)])

%% grid points
dx = 2*Lx/Nx;           % increment in physical space in x-direction
dy = 2*Ly/Ny;           % increment in physical space in y-direction

x = (-Lx):dx:(Lx-dx);   % collocation points in x
y = (-Ly):dy:(Ly-dy);   % collocation points in y

xd = x(logical((x < Lc) .* (x > -Lc))); % deep collocation points
Nd = length(xd);        % numbner of deep collocation points
NL = sum(x<=-Lc);       % number of collocation points to left of channel

%% Fourier modes
% --- wavenumbers for shallow layer ---
dkx = pi/Lx;                                % increment in wavenumbers
dky = pi/Ly;                                % increment in wavenumbers

kx = [0:(Nx/2-1), (-Nx/2):-1]*dkx;          % wavenumbers in x-direction
ky = [0:(Ny/2-1), (-Ny/2):-1]*dky;          % wavenumbers in y-direction

[KX,KY] = meshgrid(kx,ky);  
K = sqrt(KX.^2 + KY.^2);

% --- wavenumbers for deep layer ---
dxi = pi/(2*Lc);                            % increment in wavenumbers
xix = (0:(Nd-1)) * dxi;                     % wavenumbers in x direction
phij = cos((xd.' + Lc) * xix);              % basis functions in x
[XIX,XIY] = meshgrid(xix, ky);
XI = sqrt(XIX.^2 + XIY.^2);

%% fix a wavenumber ky
K_sub  = K( ky_ind, :);                     % magnitude of wavevector (shallow)
XI_sub = XI(ky_ind, :);                     % magnitude of wavevector (deep)

fm = zeros(1,Nx); fm(fm_ind) = 1;

% N.B. everywhere that an isnan term appears corrects the limiting case 
% where the wavevector has magnitude zero.

%% make matrices
% for matching condition
[kdm, xdm] = meshgrid(XI_sub,xd);
[ksm, xsm] = meshgrid(K_sub,x);

% for matching condition
prod1 = ksm .* coth(ksm * Hs); prod1(isnan(prod1)) = 1/Hs;
Asm = prod1 .* exp(1i * x.' * kx);

Adm = zeros(Nx, Nd);
Adm(NL + (1:Nd), :) = kdm .* tanh(kdm * HD) .* phij;

% consistency condition
Bsm = exp(1i * xd.' * kx);
Bdm = phij;

% coefficient of f
prod2 = ksm .* csch(ksm * Hs); prod2(isnan(prod2)) = 1/Hs;
Csm = prod2 .* exp(1i * x.' * kx);

A = [Asm, Adm; Bsm, -Bdm];

%% solve system using LU decomposition (needed for repeated solutions)
[Ldtn,Udtn,pdtn] = lu(A,'vector');
Cdtn = Csm;

bm = [Cdtn * fm.'; zeros(Nd,1)];    % RHS
xm = Udtn\(Ldtn\bm(pdtn, :));       % solution to equations
gm = xm(1:Nx, :).';                  % g (using shallow basis functions)

% ---- DtN for step ----
prod3 = K_sub .* coth(K_sub * Hs); prod3(isnan(prod3)) = 1/Hs;
prod4 = K_sub .* csch(K_sub * Hs); prod4(isnan(prod4)) = 1/Hs;
DTNm  = fm .* prod3 - gm .* prod4;

%  ---- DtN for flat bottom at two different depths ----
DTNdm = K_sub .* tanh(K_sub * Hd) .* fm;
DTNsm = K_sub .* tanh(K_sub * Hs) .* fm;

%% convert to physical space - kill infinitessimal imaginary part
% for paper I accidentally put fft instead of ifft...
shift = exp(-1i * kx * Lx );
f    = real(ifft(fm .* shift));
DTN  = real(ifft(DTNm .* shift));
DTNd = real(ifft(DTNdm .* shift));
DTNs = real(ifft(DTNsm .* shift));

%% plots
% original test function
figure
subplot(2,1,1)
plot(x, f, 'b-', 'LineWidth', 2)
grid on; box on
hold on; plot(Lc * [1 1], [-1 1]/100, 'k--', 'LineWidth', 1)
plot(-Lc * [1 1], [-1 1]/100, 'k--', 'LineWidth', 1); hold off
set(gca, 'FontSize', 18, 'FontName', 'Times New Roman')
xlabel('$$x$$', 'Interpreter', 'Latex', 'FontSize', 20)
title('Test function', 'Interpreter', 'Latex', 'FontSize', 20)

% DtN of test function for deep, shallow, and step topographies
subplot(2,1,2)
hold on
plot(x, DTNd, 'k-', 'LineWidth', 2)
plot(x, DTNs, 'r-', 'LineWidth', 2)
plot(x, DTN, 'b-', 'LineWidth', 2)
grid on; box on
plot(Lc * [1 1], [-1 1]/100, 'k--', 'LineWidth', 1)
plot(-Lc * [1 1], [-1 1]/100, 'k--', 'LineWidth', 1); hold off
set(gca, 'FontSize', 18, 'FontName', 'Times New Roman')
xlabel('$$x$$', 'Interpreter', 'Latex', 'FontSize', 20)
title('DtN of test function', 'Interpreter', 'Latex', 'FontSize', 20)
legend('DtN deep', 'DtN shallow', 'DtN step', 'Location', 'NorthEast')